from .widget import Widget
from .frame import Frame
from .stacked_widget import StackedWidget
from .splitter import Splitter, SplitterHandle, SplitterWidget
from .scroll_area import ScrollArea

from .button import Button
from .label import Label
from .line_input import LineInput
from .text_input import TextInput
from .selector import Selector
from .popup import Popup
from .statusbar import StatusBar
from .window import Window
from .slider import Slider

from .layout import Layout
from .spacer import Spacer

from PySide6.QtWidgets import QWidget
Parent = QWidget
