from PySide6.QtCore import Qt


class ElideMode:
    ElideMode = Qt.TextElideMode

    ElideRight = Qt.TextElideMode.ElideRight
    ElideLeft = Qt.TextElideMode.ElideLeft
    ElideMiddle = Qt.TextElideMode.ElideMiddle
